<?php

/**
 * Copyright (c) 2023 Zend by Perforce
 */

namespace ZRay;

use Magento\Framework\Interception\InterceptorInterface;

class Magento
{
    private array $collected = [];
    private array $events = [];

    /**
     * Send collected data to ZRay after Magento run
     */
    public function mageRunExit(array $context, array &$storage): void
    {
        if (!empty($this->events)) {
            $storage['Events'][] = array_values($this->events);
        }
        foreach ($this->collected as $key => $data) {
            $storage[$key] = [$data];
        }
    }

    /**
     * Placeholder method for ZRay plugin to intercept
     *
     * @return true
     */
    public static function zrayIntercept(
        InterceptorInterface $class,
        string $method,
        array $arguments,
        array $pluginInfo
    ): bool {
        return true;
    }

    /**
     * Collect Magento config data
     */
    public function scopeCollect(array $context, array &$storage): void
    {
        if (!array_key_exists('Config', $this->collected)) {
            $this->collected['Config'] = $context['returnValue']['stores']['default'];
        }
    }

    /**
     * Collect Magento Path data
     */
    public function pathData(array $context, array &$storage): void
    {
        if (!array_key_exists('Paths', $this->collected)) {
            $this->collected['Paths'] = $context['returnValue'];
        }
    }

    /**
     * Collect Magento Controller and Action data
     */
    public function frontController(array $context, array &$storage): void
    {
        $name = $context['functionArgs'][0] ?? null;
        if (is_object($name)) {
            $this->collected['Info']['FrontController'] = get_class($name);
            $this->collected['Info']['Action'] = str_replace(
                "\\Interceptor",
                '',
                get_class($context['locals']['actionInstance'])
            );
        }
    }

    /**
     * Collect Magento Plugin data
     */
    public function pluginsCollect(array $context, array &$storage): void
    {
        if (!is_array($context['functionArgs']) || count($context['functionArgs']) !== 6) {
            return;
        }
        list($instance, $method, $arguments, $pluginInfo, $pluginList, $type) = $context['functionArgs'];

        $valid = is_object($instance)
            && is_string($method)
            && is_array($arguments)
            && is_array($pluginInfo)
            && is_object($pluginList)
            && method_exists($pluginList, 'getPlugin')
            && is_string($type);

        if (!$valid) {
            return;
        }

        $className = substr(get_class($instance), 0, -12); // remove 'Interceptor' from the end

        $args = [];
        foreach ($arguments as $a) {
            if (is_string($a)) {
                $a = "\"" . $a . "\"";
            }
            if (is_object($a)) {
                $a = '{' . get_class($a) . '}';
            }
            if (is_array($a)) {
                $a = json_encode($a);
            }
            if (is_string($a)) {
                $len = strlen($a);
                if ($len > 40) {
                    $a = substr($a, 0, 20) . '...' . substr($a, -20);
                }
            }
            if (is_bool($a)) {
                $a = $a ? 'true' : 'false';
            }
            $args[] = $a;
        }

        $before = $pluginInfo['1'] ?? [];
        $around = $pluginInfo['2'] ?? '';
        $after = $pluginInfo['4'] ?? [];

        $plugins = [];

        $capMethod = ucfirst($method);
        if ($pluginList) {
            foreach ($before as $code) {
                $plugins[] = get_class($pluginList->getPlugin($type, $code)) . '::' . 'before' . $capMethod;
            }
            if ($around) {
                $plugins[] = get_class($pluginList->getPlugin($type, $around)) . '::' . 'around' . $capMethod;
            }
            foreach ($after as $code) {
                $plugins[] = get_class($pluginList->getPlugin($type, $code)) . '::' . 'after' . $capMethod;
            }
        }

        $this->collected['Listeners'][] = [
            'Call' => $className . '::' . $method,
            'Listener' => implode(', ', $plugins), // $plugins,
            'Arguments' => implode(', ', $args),
        ];
    }

    /**
     * Collect Magento Event data
     */
    public function eventsCollect(array $context, array &$storage): void
    {
        $event = $context['functionArgs'][0] ?? null;
        $instance = $context['this'] ?? null;
        if (empty($event) || empty($instance)) {
            return;
        }

        $observers = [];
        try {
            $mageReflect = new \ReflectionClass($instance);
            $property = $mageReflect->getProperty('_eventConfig');
            $property->setAccessible(true);
            $eventConfig = $property->getValue($context['this']);
            $observers = $eventConfig->getObservers($event);
        } catch (\Exception $e) {
        }

        unset($mageReflect);

        if (empty($observers)) {
            return;
        }

        foreach ($observers as $observer) {
            $this->events[$observer['name']] = [
                'event' => $event,
                'name' => $observer['name'],
                'observers' => $observer['instance'],
            ];
        }
    }

    /**
     * Create Interceptor mod if possible
     */
    public function magentoInit(array $context, array &$storage): void
    {
        static $init = false;
        if ($init) {
            return;
        }
        $init = true;

        $dir = BP . '/generated/code/ZRay';

        if (!file_exists($dir) || !mkdir($dir)) {
            // no access
            $this->createNoAccessListener();
            return;
        }
        $origFilename = BP . '/vendor/magento/framework/Interception/Interceptor.php';
        if (!is_readable($origFilename)) {
            $this->createNoAccessListener();
            return;
        }
        $filename = $dir . '/Interceptor.php';

        if (!file_exists($filename) || (time() - filemtime($filename) > 3600)) {
            $origInterceptor = file_get_contents($origFilename);
            $pos = strpos($origInterceptor, '___callPlugins');
            if ($pos === false) {
                $this->createNoAccessListener();
                return;
            }
            $pos = strpos($origInterceptor, '{', $pos);

            $origInterceptor = substr($origInterceptor, 0, $pos + 1)
                . "\n        "
                . "\ZRay\Magento::zrayIntercept"
                . "(\$this, \$method, \$arguments, \$pluginInfo, \$this->pluginList, \$this->subjectType);"
                . "\n"
                . substr($origInterceptor, $pos + 1);
            if (!file_put_contents($filename, $origInterceptor)) {
                // no access
                $this->createNoAccessListener();
                return;
            }
        }

        require_once $filename;

        // load plugin layout data
        if (file_exists(__DIR__ . '/meta.json')) {
            $meta = json_decode(file_get_contents(__DIR__ . '/meta.json'), true);
        } else {
            $meta = ['file does not exist' => __DIR__ . '/meta.json'];
        }
        $storage['plugin'][] = $meta;
    }

    /**
     * Create no access listener
     */
    private function createNoAccessListener(): void
    {
        $this->collected['No listeners'] = [
            'no access' => 'Please make sure PHP process can write to the folder ' . BP . '/generated/code/'
        ];
    }

    /**
     * Initialize Magento Profiler and collect version data
     */
    public function profilerInit(array $context, array &$storage): void
    {
        require_once 'zrayProfiler.php';
        \Magento\Framework\Profiler::applyConfig(
            'Zray',
            BP,
            false
        );

        $instance = $context['this'] ?? null;
        if (empty($instance) || !($instance instanceof \Magento\Framework\App\Bootstrap)) {
            return;
        }
        $objectManager = $instance->getObjectManager();

        $metaClass = $objectManager->create(\Magento\Framework\App\ProductMetadata::class);

        $this->collected['Info'] = [
            'Version' => $metaClass->getVersion(),
            'Edition' => $metaClass->getEdition(),
            'Name' => $metaClass->getName(),
            'BasePath' => BP
        ];
    }

    /**
     * Collect Magento Profiler data
     */
    public function profilerCollect(array $context, array &$storage): void
    {
        $storage['Profiler'][] = $context['this']->zrayData;
    }
}
